@@ -1,9 +1,10 @@
 ;;; browse-url.el --- Pass a URL to a WWW browser
 
-;; Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001
+;;   Free Software Foundation, Inc.
 
 ;; Author: Denis Howe <dbh@doc.ic.ac.uk>
-;; Maintainer: Matt Curtin <cmcurtin@interhack.net>
+;; Maintainer: XEmacs Development Team
 ;; Created: 03 Apr 1995
 ;; Keywords: hypertext, hypermedia, mouse
 
@@ -24,7 +25,6 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-
 ;;; Commentary:
 
 ;; This package provides functions which read a URL (Uniform Resource
@@ -33,20 +33,24 @@
 ;; URL associated with the current buffer.  Different browsers use
 ;; different methods of remote control so there is one function for
 ;; each supported browser.  If the chosen browser is not running, it
-;; is started.  Currently there is support for:
+;; is started.  Currently there is support for the following browsers,
+;; some of them probably now obsolete:
 
-;; Function              Browser     Earliest version
-;; browse-url-netscape   Netscape    1.1b1
-;; browse-url-mosaic     XMosaic/mMosaic <= 2.4
-;; browse-url-cci        XMosaic     2.5
-;; browse-url-w3         w3          0
-;; browse-url-w3-gnudoit w3 remotely
-;; browse-url-iximosaic  IXI Mosaic  ?
-;; browse-url-lynx-*	 Lynx	     0
-;; browse-url-grail      Grail       0.3b1
-;; browse-url-mmm        MMM         ?
-;; browse-url-kfm        kde
-;; browse-url-generic    arbitrary
+;; Function                           Browser     Earliest version
+;; browse-url-netscape                Netscape    1.1b1
+;; browse-url-mosaic                  XMosaic/mMosaic <= 2.4
+;; browse-url-cci                     XMosaic     2.5
+;; browse-url-w3                      w3          0
+;; browse-url-w3-gnudoit              w3 remotely
+;; browse-url-iximosaic               IXI Mosaic  ?
+;; browse-url-lynx-*	              Lynx	     0
+;; browse-url-grail                   Grail       0.3b1
+;; browse-url-mmm                     MMM         ?
+;; browse-url-generic                 arbitrary
+;; browse-url-default-windows-browser MS-Windows browser
+;; browse-url-gnome-moz               GNOME interface to Mozilla
+;; browse-url-kde                     KDE konqueror (kfm)
+;; browse-url-galeon                  Galeon
 
 ;; [A version of the Netscape browser is now free software
 ;; <URL:http://www.mozilla.org/>, albeit not GPLed, so it is
@@ -87,9 +91,8 @@
 ;; Python see <url:http://www.python.org/>.  Grail support in
 ;; browse-url.el written by Barry Warsaw <bwarsaw@python.org>.
 
-;; MMM is the freely available WWW browser implemented in Objective
-;; Caml, a cool impure functional programming language, by Francois
-;; Rouaix.  See the MMM home page
+;; MMM is a semi-free WWW browser implemented in Objective Caml, an
+;; interesting impure functional programming language.  See
 ;; <URL:http://pauillac.inria.fr/%7Erouaix/mmm/>.
 
 ;; Lynx is now distributed by the FSF.  See also
@@ -100,7 +103,8 @@
 ;; <URL:http://www.unlv.edu/chimera/>, Arena
 ;; <URL:ftp://ftp.yggdrasil.com/pub/dist/web/arena> and Amaya
 ;; <URL:ftp://ftp.w3.org/pub/amaya>.  mMosaic
-;; <URL:ftp://sig.enst.fr/pub/multicast/mMosaic/> (with development
+;; <URL:ftp://sig.enst.fr/pub/multicast/mMosaic/>,
+;; <URL:http://sig.enst.fr/~dauphin/mMosaic/> (with development
 ;; support for Java applets and multicast) can be used like Mosaic by
 ;; setting `browse-url-mosaic-program' appropriately.
 
@@ -170,16 +174,13 @@
 ;;		  (lambda ()
 ;;	             (local-set-key "\C-c\C-zf" 'browse-url-of-dired-file)))
 
-;; Browse URLs in mail messages by clicking mouse-2:
+;; Browse URLs in mail messages under RMAIL by clicking mouse-2:
 ;;	(add-hook 'rmail-mode-hook (lambda () ; rmail-mode startup
 ;;	  (define-key rmail-mode-map [mouse-2] 'browse-url-at-mouse)))
+;; Alternatively, add `goto-address' to `rmail-show-message-hook'.
 
-;; Browse URLs in Usenet messages by clicking mouse-2:
-;;	(eval-after-load "gnus"
-;;	  '(define-key gnus-article-mode-map [mouse-2] 'browse-url-at-mouse))
-;; [The current version of Gnus provides a standard feature to
-;; activate URLs in article buffers for invocation of browse-url with
-;; mouse-2.]
+;; Gnus provides a standard feature to activate URLs in article
+;; buffers for invocation of browse-url.
 
 ;; Use the Emacs w3 browser when not running under X11:
 ;;	(or (eq window-system 'x)
@@ -553,7 +554,8 @@
 
 ;;;###autoload
 (defcustom browse-url-browser-function
-  (if (eq system-type 'windows-nt)
+  (if (or (eq system-type 'windows-nt)
+	  (eq system-type 'cygwin32))
       'browse-url-default-windows-browser
     'browse-url-netscape)
   "*Function to display the current buffer in a WWW browser.
@@ -561,31 +563,35 @@ This is used by the `browse-url-at-point', `browse-url-at-mouse', and
 `browse-url-of-file' commands.
 
 If the value is not a function it should be a list of pairs
-(REGEXP . FUNCTION).  In this case the function called will be the one
+\(REGEXP . FUNCTION).  In this case the function called will be the one
 associated with the first REGEXP which matches the current URL.  The
 function is passed the URL and any other args of `browse-url'.  The last
 regexp should probably be \".\" to specify a default browser."
   :type '(choice
-           (function-item :tag "Emacs W3" :value  browse-url-w3)
-           (function-item :tag "W3 in another Emacs via `gnudoit'"
-                          :value  browse-url-w3-gnudoit)
-           (function-item :tag "Netscape" :value  browse-url-netscape)
-           (function-item :tag "Mosaic" :value  browse-url-mosaic)
-           (function-item :tag "Mosaic using CCI" :value  browse-url-cci)
-           (function-item :tag "IXI Mosaic" :value  browse-url-iximosaic)
-           (function-item :tag "Lynx in an xterm window"
-                          :value browse-url-lynx-xterm)
-           (function-item :tag "Lynx in an Emacs window"
-                          :value browse-url-lynx-emacs)
-           (function-item :tag "Grail" :value  browse-url-grail)
-           (function-item :tag "MMM" :value  browse-url-mmm)
-           (function-item :tag "kfm" :value browse-url-kfm)
-           (function-item :tag "Specified by `Browse Url Generic Program'"
-                          :value browse-url-generic)
-           (function-item :tag "Default Windows browser"
-                          :value  browse-url-default-windows-browser)
-           (function :tag "Your own function"))
-  :version "20.4"
+	  (function-item :tag "Emacs W3" :value  browse-url-w3)
+	  (function-item :tag "W3 in another Emacs via `gnudoit'"
+			 :value  browse-url-w3-gnudoit)
+	  (function-item :tag "Netscape" :value  browse-url-netscape)
+	  (function-item :tag "Mosaic" :value  browse-url-mosaic)
+	  (function-item :tag "Mosaic using CCI" :value  browse-url-cci)
+	  (function-item :tag "IXI Mosaic" :value  browse-url-iximosaic)
+	  (function-item :tag "Lynx in an xterm window"
+			 :value browse-url-lynx-xterm)
+	  (function-item :tag "Lynx in an Emacs window"
+			 :value browse-url-lynx-emacs)
+	  (function-item :tag "Grail" :value  browse-url-grail)
+	  (function-item :tag "MMM" :value  browse-url-mmm)
+	  (function-item :tag "KDE" :value browse-url-kde)
+	  (function-item :tag "Galeon" :value browse-url-galeon)
+	  (function-item :tag "Specified by `Browse Url Generic Program'"
+			 :value browse-url-generic)
+	  (function-item :tag "Default Windows browser"
+			 :value browse-url-default-windows-browser)
+	  (function-item :tag "GNOME invoking Mozilla"
+			 :value browse-url-gnome-moz)
+	  (function :tag "Your own function")
+	  (alist :tag "Regexp/function association list"
+		 :key-type regexp :value-type function))
   :group 'browse-url)
 
 (defcustom browse-url-netscape-program "netscape"
@@ -613,7 +619,7 @@ Defaults to the value of `browse-url-netscape-arguments' at the time
   :group 'browse-url)
 
 ;;;###autoload
-(defcustom browse-url-new-window-p nil
+(defcustom browse-url-new-window-flag nil
   "*If non-nil, always open a new browser window with appropriate browsers.
 Passing an interactive argument to \\[browse-url], or specific browser
 commands reverses the effect of this variable.  Requires Netscape version
@@ -621,6 +627,9 @@ commands reverses the effect of this variable.  Requires Netscape version
   :type 'boolean
   :group 'browse-url)
 
+;;;###autoload
+(defvaralias 'browse-url-new-window-p 'browse-url-new-window-flag)
+
 ;;;###autoload
 (defcustom browse-url-netscape-display nil
   "*The X display for running Netscape, if not same as Emacs'."
@@ -630,7 +639,6 @@ commands reverses the effect of this variable.  Requires Netscape version
 (defcustom browse-url-mosaic-program "xmosaic"
   "The name by which to invoke Mosaic (or mMosaic)."
   :type 'string
-  :version "20.3"
   :group 'browse-url)
 
 (defcustom browse-url-mosaic-arguments nil
@@ -644,12 +652,15 @@ commands reverses the effect of this variable.  Requires Netscape version
   :group 'browse-url)
 
 (defcustom browse-url-filename-alist
-  '(("^/\\(ftp@\\|anonymous@\\)?\\([^:]+\\):/*" . "ftp://\\2/")
+  (\`					; Backquote syntax won't work.
+   (("^/\\(ftp@\\|anonymous@\\)?\\([^:]+\\):/*" . "ftp://\\2/")
     ;; The above loses the username to avoid the browser prompting for
     ;; it in anonymous cases.  If it's not anonymous the next regexp
     ;; applies.
     ("^/\\([^:@]+@\\)?\\([^:]+\\):/*" . "ftp://\\1\\2/")
-    ("^/+" . "file:/"))
+    (,@ (if (memq system-type '(windows-nt ms-dos))
+	    '(("^\\([a-zA-Z]:\\)[\\/]" . "file:\\1/"))))
+    ("^/+" . "file:/")))
   "An alist of (REGEXP . STRING) pairs used by `browse-url-of-file'.
 Any substring of a filename matching one of the REGEXPs is replaced by
 the corresponding STRING using `replace-match', not treating STRING
@@ -670,7 +681,6 @@ address to an HTTP URL:
   :type '(repeat (cons :format "%v"
                        (regexp :tag "Regexp")
                        (string :tag "Replacement")))
-  :version "20.3"
   :group 'browse-url)
 
 ;;;###autoload
@@ -727,7 +737,7 @@ These might set its size, for instance."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
-(defcustom browse-url-lynx-emacs-args (and (not window-system) 
+(defcustom browse-url-lynx-emacs-args (and (not window-system)
                                            '("-show_cursor"))
   "A list of strings defining options for Lynx in an Emacs buffer.
 
@@ -735,7 +745,6 @@ The default is none in a window system, otherwise `-show_cursor' to
 indicate the position of the current link in the absence of
 highlighting, assuming the normal default for showing the cursor."
   :type '(repeat (string :tag "Argument"))
-  :version "20.3"
   :group 'browse-url)
 
 (defcustom browse-url-gnudoit-program "gnuclient"
@@ -787,7 +796,6 @@ down (this *won't* always work)."
   :type '(choice (const :tag "Move to try to avoid field" :value avoid)
                  (const :tag "Disregard" :value nil)
                  (const :tag "Warn, don't emit URL" :value warn))
-  :version "20.3"
   :group 'browse-url)
 
 (defvar browse-url-lynx-input-attempts 10
@@ -798,13 +806,22 @@ down (this *won't* always work)."
 
 (defvar browse-url-temp-file-list '())
 
+(defcustom browse-url-kde-program "kfmclient"
+  "The name by which to invoke the KDE web browser."
+  :type 'string
+  :group 'browse-url)
+
+(defcustom browse-url-kde-args '("openURL")
+  "*A list of strings defining options for `browse-url-kde-program'."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; URL input
 
 (defun browse-url-url-at-point ()
   (let ((url (thing-at-point 'url)))
-    (when (featurep 'xemacs)
-      (set-text-properties 0 (length url) nil url))
+    (set-text-properties 0 (length url) nil url)
     url))
 
 ;; Having this as a separate function called by the browser-specific
@@ -815,14 +832,21 @@ down (this *won't* always work)."
   "Read a URL from the minibuffer, prompting with PROMPT.
 Default to the URL at or before point.  If invoked with a mouse button,
 set point to the position clicked first.  Return a list for use in
-`interactive' containing the URL and `browse-url-new-window-p' or its
+`interactive' containing the URL and `browse-url-new-window-flag' or its
 negation if a prefix argument was given."
   (let ((event (elt (this-command-keys) 0)))
     (and (listp event) (mouse-set-point event)))
   (list (read-string prompt (browse-url-url-at-point))
-	(not (eq (null browse-url-new-window-p)
+	(not (eq (null browse-url-new-window-flag)
 		 (null current-prefix-arg)))))
 
+;; interactive-p needs to be called at a function's top-level, hence
+;; the macro.
+(defmacro browse-url-maybe-new-window (arg)
+  `(if (interactive-p)
+       ,arg
+     browse-url-new-window-flag))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Browse current buffer
 
@@ -858,14 +882,9 @@ Use variable `browse-url-filename-alist' to map filenames to URLs."
   (while (string-match "[*\"()',=;? ]" file)
     (let ((enc (format "%%%x" (aref file (match-beginning 0)))))
       (setq file (replace-match enc t t file))))
-  (let ((maps browse-url-filename-alist))
-    (while maps
-      (let* ((map (car maps))
-	     (from-re (car map))
-	     (to-string (cdr map)))
-	(setq maps (cdr maps))
-	(and (string-match from-re file)
-	     (setq file (replace-match to-string t nil file))))))
+  (dolist (map browse-url-filename-alist)
+    (when (and map (string-match (car map) file))
+      (setq file (replace-match (cdr map) t nil file))))
   file)
 
 ;;;###autoload
@@ -887,7 +906,7 @@ narrowed."
 	    (or browse-url-temp-file-name
 		(setq browse-url-temp-file-name
 		      (convert-standard-filename
-		       (make-temp-name
+		       (make-temp-name ;; XEmacs doesn't have make-temp-file
 			(expand-file-name "burl" browse-url-temp-dir)))))
 	    (setq file-name browse-url-temp-file-name)
 	    (write-region (point-min) (point-max) file-name nil 'no-message)))
@@ -943,32 +962,32 @@ narrowed."
 Prompts for a URL, defaulting to the URL at or before point.  Variable
 `browse-url-browser-function' says which browser to use."
   (interactive (browse-url-interactive-arg "URL: "))
-  (let ((bf browse-url-browser-function) re)
-    (unless (functionp bf)
-      (while (consp bf)
-	(setq re (car (car bf))
-	      bf (if (string-match re url)
-		     (cdr (car bf))	; The function
-		   (cdr bf)))))		; More pairs
-    (or bf (error "No browser in browse-url-browser-function matching URL %s"
-                  url))
-    (apply bf url args)))
+  (unless (interactive-p)
+    (setq args (or args (list browse-url-new-window-flag))))
+  (if (functionp browse-url-browser-function)
+      (apply browse-url-browser-function url args)
+    ;; The `function' can be an alist; look down it for first match
+    ;; and apply the function (which might be a lambda).
+    (catch 'done
+      (dolist (bf browse-url-browser-function)
+	(when (string-match (car bf) url)
+	  (apply (cdr bf) url args)
+	  (throw 'done t)))
+      (error "No browse-url-browser-function matching URL %s"
+	     url))))
 
 ;;;###autoload
-(defun browse-url-at-point ()
+(defun browse-url-at-point (&optional arg)
   "Ask a WWW browser to load the URL at or before point.
 Doesn't let you edit the URL like `browse-url'.  Variable
 `browse-url-browser-function' says which browser to use."
-  (interactive)
-  (browse-url (browse-url-url-at-point)))
-
-; XEmacs change
-(defun browse-url-event-buffer (event)
-  (event-buffer event))
-
-; XEmacs change
-(defun browse-url-event-point (event)
-  (event-point event))
+  (interactive "P")
+  (let ((url (browse-url-url-at-point)))
+    (if url
+	(browse-url url (if arg
+			    (not browse-url-new-window-flag)
+			  browse-url-new-window-flag))
+      (error "No URL found"))))
 
 ;;;###autoload
 (defun browse-url-at-mouse (event)
@@ -979,12 +998,8 @@ but point is not changed.  Doesn't let you edit the URL like
 to use."
   (interactive "e")
   (save-excursion
-    (set-buffer (browse-url-event-buffer event))
-    (goto-char (browse-url-event-point event))
-    (let ((url (browse-url-url-at-point)))
-      (if (string-equal url "")
-	  (error "No URL found"))
-      (browse-url url browse-url-new-window-p))))
+    (mouse-set-point event)
+    (browse-url-at-point browse-url-new-window-flag)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Browser-specific commands
@@ -1022,18 +1037,17 @@ one showing the selected frame."
 ;;;###autoload
 (defun browse-url-netscape (url &optional new-window)
   "Ask the Netscape WWW browser to load URL.
-
 Default to the URL around or before point.  The strings in variable
 `browse-url-netscape-arguments' are also passed to Netscape.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil, load the document in a new Netscape window, otherwise use a
 random existing one.  A non-nil interactive prefix argument reverses
-the effect of `browse-url-new-window-p'.
+the effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
-  (interactive (browse-url-interactive-arg "Netscape URL: "))
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))
   ;; URL encode any `confusing' characters in the URL.  This needs to
   ;; include at least commas; presumably also close parens.
   (while (string-match "[,)]" url)
@@ -1051,11 +1065,13 @@ used instead of `browse-url-new-window-p'."
 			     (if new-window '("-noraise"))
 			     (list "-remote"
 				   (concat "openURL(" url
-					   (if new-window ",new-window")
+					   (if (browse-url-maybe-new-window
+						new-window)
+					       ",new-window")
 					   ")"))))))))
     (set-process-sentinel process
-			  (list 'lambda '(process change)
-				(list 'browse-url-netscape-sentinel 'process url)))))
+			  `(lambda (process change)
+			     (browse-url-netscape-sentinel process ,url)))))
 
 (defun browse-url-netscape-sentinel (process url)
   "Handle a change to the process communicating with Netscape."
@@ -1085,6 +1101,36 @@ How depends on `browse-url-netscape-version'."
            (append browse-url-netscape-arguments
                    (list "-remote" command)))))
 
+;; GNOME means of invoking either Mozilla or Netrape.
+
+(defcustom browse-url-gnome-moz-arguments '()
+  "A list of strings passed to the GNOME mozilla viewer as arguments."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+;;;###autoload
+(defun browse-url-gnome-moz (url &optional new-window)
+  "Ask Mozilla/Netscape to load URL via the GNOME program `gnome-moz-remote'.
+Default to the URL around or before point.  The strings in variable
+`browse-url-gnome-moz-arguments' are also passed.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new browser window, otherwise use an
+existing one.  A non-nil interactive prefix argument reverses the
+effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))  
+  (apply 'start-process (concat "gnome-moz-remote " url)
+	 nil
+	 "gnome-moz-remote"
+	 (append
+	  browse-url-gnome-moz-arguments
+	  (if (browse-url-maybe-new-window new-window)
+	    '("--newwin"))
+	  (list "--raise" url))))
+
 ;; --- Mosaic ---
 
 ;;;###autoload
@@ -1096,13 +1142,13 @@ Default to the URL around or before point.  The strings in variable
 program is invoked according to the variable
 `browse-url-mosaic-program'.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil, load the document in a new Mosaic window, otherwise use a
 random existing one.  A non-nil interactive prefix argument reverses
-the effect of `browse-url-new-window-p'.
+the effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
+used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Mosaic URL: "))
   (let ((pidfile (expand-file-name browse-url-mosaic-pidfile))
 	pid)
@@ -1116,7 +1162,7 @@ used instead of `browse-url-new-window-p'."
 	(save-excursion
 	  (find-file (format "/tmp/Mosaic.%d" pid))
 	  (erase-buffer)
-	  (insert (if new-window
+	  (insert (if (browse-url-maybe-new-window new-window)
 		      "newwin\n"
 		    "goto\n")
 		  url "\n")
@@ -1154,7 +1200,7 @@ variable `browse-url-grail'."
     (set-buffer (get-buffer-create " *Shell Command Output*"))
     (erase-buffer)
     ;; don't worry about this failing.
-    (if new-window
+    (if (browse-url-maybe-new-window new-window)
 	(call-process browse-url-grail nil 0 nil "-b" url)
       (call-process browse-url-grail nil 0 nil url))
     (message "Sending URL to Grail... done")))
@@ -1170,20 +1216,20 @@ This function only works for XMosaic version 2.5 or later.  You must
 select `CCI' from XMosaic's File menu, set the CCI Port Address to the
 value of variable `browse-url-CCI-port', and enable `Accept requests'.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil, load the document in a new browser window, otherwise use a
 random existing one.  A non-nil interactive prefix argument reverses
-the effect of `browse-url-new-window-p'.
+the effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
+used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Mosaic URL: "))
   (open-network-stream "browse-url" " *browse-url*"
 		       browse-url-CCI-host browse-url-CCI-port)
   ;; Todo: start browser if fails
   (process-send-string "browse-url"
 		       (concat "get url (" url ") output "
-			       (if new-window
+			       (if (browse-url-maybe-new-window new-window)
 				   "new"
 				 "current")
 			       "\r\n"))
@@ -1208,15 +1254,15 @@ Default to the URL around or before point."
   "Ask the w3 WWW browser to load URL.
 Default to the URL around or before point.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil, load the document in a new window.  A non-nil interactive
-prefix argument reverses the effect of `browse-url-new-window-p'.
+prefix argument reverses the effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
+used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "W3 URL: "))
   (require 'w3)				; w3-fetch-other-window not autoloaded
-  (if new-window
+  (if (browse-url-maybe-new-window new-window)
       (w3-fetch-other-window url)
     (w3-fetch url)))
 
@@ -1229,7 +1275,9 @@ The `browse-url-gnudoit-program' program is used with options given by
   (interactive (browse-url-interactive-arg "W3 URL: "))
     (apply 'start-process (concat "gnudoit:" url) nil
 	   browse-url-gnudoit-program
-	   (append browse-url-gnudoit-args (list (concat "(w3-fetch \"" url "\")") "(raise-frame)"))))
+	   (append browse-url-gnudoit-args
+		   (list (concat "(w3-fetch \"" url "\")")
+			 "(raise-frame)"))))
 
 ;; --- Lynx in an xterm ---
 
@@ -1242,7 +1290,8 @@ in an Xterm window using the Xterm program named by `browse-url-xterm-program'
 with possible additional arguments `browse-url-xterm-args'."
   (interactive (browse-url-interactive-arg "Lynx URL: "))
   (apply #'start-process `(,(concat "lynx" url) nil ,browse-url-xterm-program
-             ,@browse-url-xterm-args "-e" "lynx" ,url)))
+             ,@browse-url-xterm-args "-e" "lynx"
+	     ,url)))
 
 ;; --- Lynx in an Emacs "term" window ---
 
@@ -1252,25 +1301,26 @@ with possible additional arguments `browse-url-xterm-args'."
 Default to the URL around or before point.  With a prefix argument, run
 a new Lynx process in a new buffer.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil, load the document in a new lynx in a new term window,
 otherwise use any existing one.  A non-nil interactive prefix argument
-reverses the effect of `browse-url-new-window-p'.
+reverses the effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
+used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Lynx URL: "))
   (let* ((system-uses-terminfo t)       ; Lynx uses terminfo
+	 ;; (term-term-name "vt100") ; ??
 	 (buf (get-buffer "*lynx*"))
 	 (proc (and buf (get-buffer-process buf)))
 	 (n browse-url-lynx-input-attempts))
-    (if (and new-buffer buf)
+    (if (and (browse-url-maybe-new-window new-buffer) buf)
 	;; Rename away the OLD buffer. This isn't very polite, but
 	;; term insists on working in a buffer named *lynx* and would
 	;; choke on *lynx*<1>
 	(progn (set-buffer buf)
 	       (rename-uniquely)))
-    (if (or new-buffer
+    (if (or (browse-url-maybe-new-window new-buffer)
 	    (not buf)
 	    (not proc)
 	    (not (memq (process-status proc) '(run stop))))
@@ -1278,7 +1328,8 @@ used instead of `browse-url-new-window-p'."
 	(progn
           (setq buf
                 (apply #'make-term
-                       `("lynx" "lynx" nil ,@browse-url-lynx-emacs-args ,url)))
+                       `("lynx" "lynx" nil ,@browse-url-lynx-emacs-args
+			 ,url)))
           (switch-to-buffer buf)
           (term-char-mode)
           (set-process-sentinel
@@ -1337,19 +1388,19 @@ recipient's address.  Supplying a non-nil interactive prefix argument
 will cause the mail to be composed in another window rather than the
 current one.
 
-When called interactively, if variable `browse-url-new-window-p' is
+When called interactively, if variable `browse-url-new-window-flag' is
 non-nil use `compose-mail-other-window', otherwise `compose-mail'.  A
 non-nil interactive prefix argument reverses the effect of
-`browse-url-new-window-p'.
+`browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
-used instead of `browse-url-new-window-p'."
+used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Mailto URL: "))
   (save-excursion
     (let ((to (if (string-match "^mailto:" url)
 		  (substring url 7)
 		url)))
-      (if new-window
+      (if (browse-url-maybe-new-window new-window)
 	  (compose-mail-other-window to nil nil nil
 				     (list 'insert-buffer (current-buffer)))
 	(compose-mail to nil nil nil nil
@@ -1373,12 +1424,32 @@ don't offer a form of remote control."
 	 (append browse-url-generic-args (list url))))
 
 ;;;###autoload
-(defun browse-url-kfm (url &optional new-window)
-  "Ask the kfm WWW browser to load URL.
+(defun browse-url-kde (url &optional new-window)
+  "Ask the KDE WWW browser to load URL.
 Default to the URL around or before point."
-  (interactive (browse-url-interactive-arg "kfm URL: "))
-  (message "Sending URL to kfm...")
-    (start-process "kfmclient" nil "kfmclient"	 "openURL" url))
+  (interactive (browse-url-interactive-arg "KDE URL: "))
+  (message "Sending URL to KDE...")
+  (apply #'start-process `(,(concat "KDE" url) nil ,browse-url-kde-program
+			   ,@browse-url-kde-args ,url)))
+
+(defalias 'browse-url-kfm 'browse-url-kde) ;; XEmacs backwards compatibility
+
+;; --- Galeon ---
+
+;;;###autoload
+(defun browse-url-galeon (url &optional new-window)
+  "Ask the Galeon WWW browser to load URL.
+Default to the URL around or before point."
+  (interactive (browse-url-interactive-arg "Galeon URL: "))
+  (message "Sending URL to Galeon...")
+  (save-excursion
+    (set-buffer (get-buffer-create " *Shell Command Output*"))
+    (erase-buffer)
+    ;; don't worry about this failing.
+    (if (browse-url-maybe-new-window new-window)
+	(call-process "galeon" nil 0 nil "-w" url)
+      (call-process "galeon" nil 0 nil "-x" url))
+    (message "Sending URL to Galeon... done")))
 
 (provide 'browse-url)
 
