@@ -1,14 +1,15 @@
 ;;; smtpmail.el --- simple SMTP protocol (RFC 821) for sending mail
-;;; ###	Hacked by Mike Taylor, 11th October 1999 to add support for
-;;;	automatically appending a domain to RCPT TO: addresses.
 
-;; Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+;; Copyright (C) 1995, 1996, 2001 Free Software Foundation, Inc.
 
 ;; Author: Tomoji Kagatani <kagatani@rbc.ncl.omron.co.jp>
 ;; Maintainer: Brian D. Carlstrom <bdc@ai.mit.edu>
 ;; ESMTP support: Simon Leinen <simon@switch.ch>
+;; Hacked by Mike Taylor, 11th October 1999 to add support for
+;; automatically appending a domain to RCPT TO: addresses.
 ;; AUTH=LOGIN support: Stephen Cranefield <scranefield@infoscience.otago.ac.nz>
 ;; AUTH support: Simon Josefsson <jas@pdc.kth.se>
+;; STARTTLS support: Simon Josefsson <jas@pdc.kth.se>
 ;; Keywords: mail
 
 ;; This file is part of XEmacs.
@@ -28,7 +29,7 @@
 ;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 ;; 02111-1307, USA.
 
-;;; Synched up with: FSF 20.7
+;;; Synched up with: FSF 21.0.104
 
 ;;; Commentary:
 
@@ -121,12 +122,6 @@ buffer includes an exchange like:
   :type '(choice (const nil) string)
   :group 'smtpmail)
 
-(defun maybe-append-domain (recipient)
-  (if (or (not smtpmail-sendto-domain)
-	  (string-match "@" recipient))
-      recipient
-    (concat recipient "@" smtpmail-sendto-domain)))
-
 (defcustom smtpmail-debug-info nil
   "*smtpmail debug info printout. messages and process buffer."
   :type 'boolean
@@ -172,6 +167,13 @@ This is a list of four-element list with `servername' (a string),
 		       (file    :tag "Certificate")))
   :group 'smtpmail)
 
+(defcustom smtpmail-warn-about-unknown-extensions nil
+  "*If set, print warnings about unknown SMTP extensions.
+This is mainly useful for development purposes, to learn about
+new SMTP extensions that might be useful to support."
+  :type 'boolean
+  :group 'smtpmail)
+
 (defvar smtpmail-queue-index-file "index"
   "File name of queued mail index,
 This is relative to `smtpmail-queue-dir'.")
@@ -185,7 +187,7 @@ This is relative to `smtpmail-queue-dir'.")
 (defvar smtpmail-queue-index (concat smtpmail-queue-dir
 				     smtpmail-queue-index-file))
 
-(defconst smtpmail-auth-supported '(cram-md5)
+(defconst smtpmail-auth-supported '(cram-md5 login)
   "List of supported SMTP AUTH mechanisms.")
 
 ;;;
@@ -302,9 +304,9 @@ This is relative to `smtpmail-queue-dir'.")
 	  ;;
 	  (setq smtpmail-address-buffer (generate-new-buffer "*smtp-mail*"))
 	  (setq smtpmail-recipient-address-list
-		(smtpmail-deduce-address-list tembuf (point-min) delimline))
+		    (smtpmail-deduce-address-list tembuf (point-min) delimline))
 	  (kill-buffer smtpmail-address-buffer)
-
+	  
 	  ;; XEmacs change
 	  (mail-check-safe-charset)
 
@@ -395,6 +397,12 @@ This is relative to `smtpmail-queue-dir'.")
 		   (equal port (nth 1 el)))
 	  (throw 'done el))))))
 
+(defun smtpmail-maybe-append-domain (recipient)
+  (if (or (not smtpmail-sendto-domain)
+	  (string-match "@" recipient))
+      recipient
+    (concat recipient "@" smtpmail-sendto-domain)))
+
 (defun smtpmail-via-smtp (recipient smtpmail-text-buffer)
   (let ((process nil)
 	(host (or smtpmail-smtp-server
@@ -441,53 +449,61 @@ This is relative to `smtpmail-queue-dir'.")
 	    (make-local-variable 'smtpmail-read-point)
 	    (setq smtpmail-read-point (point-min))
 
+	    
 	    (if (or (null (car (setq greeting (smtpmail-read-response process))))
 		    (not (integerp (car greeting)))
 		    (>= (car greeting) 400))
 		(throw 'done nil)
 	      )
 
-	    ;; EHLO
-	    (smtpmail-send-command process (format "EHLO %s" (smtpmail-fqdn)))
+	    (let ((do-ehlo t)
+		  (do-starttls t))
+	      (while do-ehlo
+		;; EHLO
+		(smtpmail-send-command process (format "EHLO %s" (smtpmail-fqdn)))
 
-	    (if (or (null (car (setq response-code (smtpmail-read-response process))))
-		    (not (integerp (car response-code)))
-		    (>= (car response-code) 400))
-		(progn
-		  ;; HELO
-		  (smtpmail-send-command process (format "HELO %s" (smtpmail-fqdn)))
+		(if (or (null (car (setq response-code (smtpmail-read-response process))))
+			(not (integerp (car response-code)))
+			(>= (car response-code) 400))
+		    (progn
+		      ;; HELO
+		      (smtpmail-send-command process (format "HELO %s" (smtpmail-fqdn)))
 
-		  (if (or (null (car (setq response-code (smtpmail-read-response process))))
-			  (not (integerp (car response-code)))
-			  (>= (car response-code) 400))
-		      (throw 'done nil)))
-	      (let ((extension-lines (cdr (cdr response-code))))
-		(while extension-lines
-		  (let ((name (mapcar 'intern (mapcar 'downcase (split-string (substring (car extension-lines) 4) "[ ]")))))
-		    (and (eq (length name) 1)
-			 (setq name (car name)))
-		    (and name
-			 (if (memq (if (consp name) (car name) name)
-				   '(verb xvrb 8bitmime onex xone
-					  expn size dsn etrn 
-					  enhancedstatuscodes
-					  help xusr
-					  auth=login auth starttls))
-			     (setq supported-extensions
-				   (cons name supported-extensions))
-			   (message "unknown extension %s" name))))
-		  (setq extension-lines (cdr extension-lines)))))
+		      (if (or (null (car (setq response-code (smtpmail-read-response process))))
+			      (not (integerp (car response-code)))
+			      (>= (car response-code) 400))
+			  (throw 'done nil)))
+		  (let ((extension-lines (cdr (cdr response-code))))
+		    (while extension-lines
+		      (let ((name (mapcar 'intern (mapcar 'downcase (split-string (substring (car extension-lines) 4) "[ ]")))))
+			(and (eq (length name) 1)
+			     (setq name (car name)))
+			(and name
+			     (cond ((memq (if (consp name) (car name) name)
+					  '(verb xvrb 8bitmime onex xone
+						 expn size dsn etrn 
+						 enhancedstatuscodes
+						 help xusr
+						 auth=login auth starttls))
+				    (setq supported-extensions
+					  (cons name supported-extensions)))
+				   (smtpmail-warn-about-unknown-extensions
+				    (message "Unknown extension %s" name)))))
+		      (setq extension-lines (cdr extension-lines)))))
 	  
-	    (if (and (smtpmail-find-credentials smtpmail-starttls-credentials host port)
-		     (member 'starttls supported-extensions)
-		     (process-id process))
-		(progn
-		  (smtpmail-send-command process (format "STARTTLS"))
-		  (if (or (null (car (setq response-code (smtpmail-read-response process))))
-			  (not (integerp (car response-code)))
-			  (>= (car response-code) 400))
-		      (throw 'done nil))
-		  (starttls-negotiate process)))
+		(if (and do-starttls
+			 (smtpmail-find-credentials smtpmail-starttls-credentials host port)
+			 (member 'starttls supported-extensions)
+			 (process-id process))
+		    (progn
+		      (smtpmail-send-command process (format "STARTTLS"))
+		      (if (or (null (car (setq response-code (smtpmail-read-response process))))
+			      (not (integerp (car response-code)))
+			      (>= (car response-code) 400))
+			  (throw 'done nil))
+		      (starttls-negotiate process)
+		      (setq do-starttls nil))
+		  (setq do-ehlo nil))))
 
 	    (let* ((mechs (cdr-safe (assoc 'auth supported-extensions)))
 		   (mech (car (intersection smtpmail-auth-supported mechs)))
@@ -510,7 +526,7 @@ This is relative to `smtpmail-queue-dir'.")
 				   (not (integerp (car response-code)))
 				   (>= (car response-code) 400))
 			       (throw 'done nil)))))
-		      ((member 'auth=login supported-extensions)
+		      ((eq mech 'login)
 		       (smtpmail-send-command process "AUTH LOGIN")
 		       (if (or (null (car (setq response-code (smtpmail-read-response process))))
 			       (not (integerp (car response-code)))
@@ -593,7 +609,9 @@ This is relative to `smtpmail-queue-dir'.")
 		     "")))
 ;	      (smtpmail-send-command process (format "MAIL FROM:%s@%s" (user-login-name) (smtpmail-fqdn)))
 	      (smtpmail-send-command process (format "MAIL FROM: <%s>%s%s"
-						     (user-mail-address) ;; XEmacs
+						     ;; XEmacs:
+						     (or mail-envelope-from
+							 (user-mail-address))
 						     size-part
 						     body-part))
 
@@ -606,7 +624,7 @@ This is relative to `smtpmail-queue-dir'.")
 	    ;; RCPT TO: <recipient>
 	    (let ((n 0))
 	      (while (not (null (nth n recipient)))
-		(smtpmail-send-command process (format "RCPT TO: <%s>" (maybe-append-domain (nth n recipient))))
+		(smtpmail-send-command process (format "RCPT TO: <%s>" (smtpmail-maybe-append-domain (nth n recipient))))
 		(setq n (1+ n))
 
 		(setq response-code (smtpmail-read-response process))
